# -*- ruby -*-

require 'xtemplate/xml'

module XTemplate
  class HTMLParser < XMLParser
    def parse(str)
      # completing tags
      raise(NotImplementedError)
      super(comp_str)
    end
  end

  class HTMLDocument < XMLDocument
    def parse(text)
      HTMLParser.new.parse(text)
    end
  end
end
