require 'xtemplate'
$KCODE = "EUC-JP"

template = XTemplate::XMLTemplate.new(<<EOS)
<xt:template xmlns:xt="http://xtemplate.sourceforge.net/xtemplate">
  <xt:copy-of id="{import('xmlrpc://rnn.sourceforge.jp/cgi-bin/xmlrpc','rnn.list',0,2,-1)}" />
</xt:template>
EOS

template.expand2($stdout,{})
$stdout << "\n"
